﻿-----------------------------------------------------------------------------
--
-- data
--   \activity
--     \<playerName>
--       - total
--       - damage
--       - healing
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentActivity")
L:RegisterTranslations("enUS", function() return {
	["Activity"] = true,
	["Active List"] = true,
	["Damage Activity"] = true,
	["Healing Activity"] = true,
	["Activity Options."] = true,
	["Activity Data"] = true,
	["Gathers information on player activity."] = true,
	["Merge Pet Activity"] = true,
	["Merges pets activity with their owner's data."] = true,
	["Average Instant DPS"] = true,
	["Average Instant HPS"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Activity"] = "动作",
	["Active List"] = "动作列表",
	["Damage Activity"] = "伤害动作",
	["Healing Activity"] = "治疗动作",
	["Activity Options."] = "动作选项",
	["Activity Data"] = "动作数据",
	["Gathers information on player activity."] = "收集玩家的动作信息",
	["Merge Pet Activity"] = "合并宠物动作",
	["Merges pets activity with their owner's data."] = "将宠物动作合并到主人的数据中",
	["Average Instant DPS"] = "平均DPS",
	["Average Instant HPS"] = "平均HPS",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Activity"] = "動作",
	["Active List"] = "動作列表",
	["Damage Activity"] = "傷害動作",
	["Healing Activity"] = "治療動作",
	["Activity Options."] = "動作選項",
	["Activity Data"] = "動作數據",
	["Gathers information on player activity."] = "收集玩家的動作信息",
	["Merge Pet Activity"] = "合併寵物動作",
	["Merges pets activity with their owner's data."] = "將寵物動作合併到主人的數據中",
	["Average Instant DPS"] = "平均DPS",
	["Average Instant HPS"] = "平均HPS",
} end)

L:RegisterTranslations("frFR", function() return {
	["Activity"] = "Activit\195\169",
	["Active List"] = "Liste d'Activit\195\169",
	["Damage Activity"] = "Dommage Actif",
	["Healing Activity"] = "Soin Actif",
	["Activity Options."] = "Options d'Activit\195\169.",
	["Activity Data"] = "Donn\195\169e d'Activit\195\169",
	["Gathers information on player activity."] = "Analyse les informations de l'activit\195\169 des joueurs",
	["Merge Pet Activity"] = "Fusion des Familiers",
	["Merges pets activity with their owner's data."] = "Fusione les informations des familiers avec ceux de leur ma\195\174tre",
	["Average Instant DPS"] = "DPS moyen instantan\195\169",
	["Average Instant HPS"] = "HPS moyen instantan\195\169",
} end)

L:RegisterTranslations("koKR", function() return {
	["Activity"] = "활성시간",
	["Active List"] = "활성시간 리스트",
	["Damage Activity"] = "공격 활성시간",
	["Healing Activity"] = "치유 활성화",
	["Activity Options."] = "활성시간 설정",
	["Activity Data"] = "활성시간 데이터",
	["Gathers information on player activity."] = "플레이어의 전투 활성시간 정보를 수집합니다.",
	["Merge Pet Activity"] = "소환수 활성시간 합치기",
	["Merges pets activity with their owner's data."] = "소환수의 활성시간도 캐릭터의 활성시간에 합칩니다.",
	["Average Instant DPS"] = "평균 순간 DPS",
	["Average Instant HPS"] = "평균 순간 HPS",
} end)

-----------------------------------------------------------------------------
-- The following is used to avoid relocalizing damage/healing details display set names
local LD = AceLibrary("AceLocale-2.2"):new("AssessmentDamage")
local LH = AceLibrary("AceLocale-2.2"):new("AssessmentHealing")

-----------------------------------------------------------------------------

local AssessmentActivity = Assessment:NewModule("Activity", "AceEvent-2.0", "Parser-3.0")
AssessmentActivity.db = Assessment:AcquireDBNamespace("Activity")
Assessment:RegisterDefaults("Activity", "profile", {
	enable												= true,
	mergePetWithOwner							= true,
	gatherActivityDataFilter			= "ALL",
})

-----------------------------------------------------------------------------

local profile
local gatherActivityDataFilter
local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable
local pairs = pairs
local min, max = math.min, math.max
local GetTime = GetTime

-----------------------------------------------------------------------------

Assessment.TagData.Activity =
{
	method = function(data)
		local fightData = data.fightData
		if not fightData then return 0 end
		
		local activity = fightData.activity
		if not activity then return 0 end
		
		local actor = activity[data.key]
		if not actor then return 0 end
	
		return actor.total
	end,
	data = "activity",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.TagData.DamageActivity =
{
	method = function(data)
		local fightData = data.fightData
		if not fightData then return 0 end
		
		local activity = fightData.activity
		if not activity then return 0 end
		
		local actor = activity[data.key]
		if not actor then return 0 end
	
		return actor.damage
	end,
	data = "activity",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.TagData.HealingActivity =
{
	method = function(data)
		local fightData = data.fightData
		if not fightData then return 0 end
		
		local activity = fightData.activity
		if not activity then return 0 end
		
		local actor = activity[data.key]
		if not actor then return 0 end
	
		return actor.healing
  end,
	data = "activity",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.ActiveList =
{
	method = function(fightData)
		if not fightData or not fightData.activity then return emptyMethod end
		return pairs(fightData.activity)
	end,
	data = "activity",
	description = L["Active List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.ActivityList =
{
	titleTag					= "Activity List",
	listType					= "ActiveList", 
	barValueTag				= "Activity", 
	filterTag					= "BarValue ~= 0",
	sortKeyTag				= "BarValue",
	fightIndex				= 0,
	text							= {
		{ tag = "[Rank]. [Key]" },
		{ tag = "[BarValue:FormatDuration]" },
		count = 2
	},
	description				= L["Activity"],
	segmentEnable			= false,
	defaultColor			= { 0.5, 0.5, 0.3 },
	colorTypeList			= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.DamageActivity =
{
	titleTag					= "Damage Activity",
	listType					= "ActiveList", 
	barValueTag				= "DamageActivity", 
	filterTag					= "BarValue ~= 0",
	sortKeyTag				= "BarValue",
	fightIndex				= 0,
	text							= {
		{ tag = "[Rank]. [Key]" },
		{ tag = "[BarValue:FormatDuration]" },
		count = 2
	},
	description				= L["Damage Activity"],
	segmentEnable			= false,
	defaultColor			= { 0.8, 0.3, 0.3 },
	colorTypeList			= { "Class", count = 1 },
	favorite				= false,
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.HealingActivity =
{
	titleTag					= "Healing Activity",
	listType					= "ActiveList", 
	barValueTag				= "HealingActivity", 
	filterTag					= "BarValue ~= 0",
	sortKeyTag				= "BarValue",
	fightIndex				= 0,
	text							= {
		{ tag = "[Rank]. [Key]" },
		{ tag = "[BarValue:FormatDuration]" },
		count = 2
	},
	description				= L["Healing Activity"],
	segmentEnable			= false,
	defaultColor			= { 0.3, 0.8, 0.3 },
	colorTypeList			= { "Class", count = 1 },
	favorite				= false,
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.AverageInstantDPS =
{
	titleTag					= "[TotalBarValue:Integer]dps ([CombatDuration:FormatDuration])",
	listType					= "ActiveList", 
	barValueTag				= "EnemyDamageDealt / Maximum(DamageActivity, 1)", 
	filterTag					= "BarValue ~= 0",
	sortKeyTag				= "BarValue",
	fightIndex				= 1,
	text							= {
		{ tag = "[Rank]. [Key]" },
		{ tag= "[BarValue:Integer]" },
		{ tag = "[(BarValue/TotalBarValue):Percent]", visible = false },
		count = 3
	},
	description				= L["Average Instant DPS"],
	onClickType				= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= string.format("[Select(IsControlKeyDown, %q, %q)]", LD["Damage Type Dealt Details"], LD["Damage Dealt Details"]),
	onClickData				= "[Key]",
	segmentEnable			= false,
	segmentDisplaySet	= LD["Damage Type"],
	segmentData				= "damageDealt,[Key]",
	defaultColor			= { 0.8, 0.8, 0.3 },
	colorTypeList			= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.AverageInstantHPS =
{
	titleTag					= "[TotalBarValue:Integer]hps ([CombatDuration:FormatDuration])",
	listType					= "ActiveList", 
	barValueTag				= "(HealingDone - OverhealingDone) / Maximum(HealingActivity, 1)", 
	filterTag					= "BarValue ~= 0",
	sortKeyTag				= "BarValue",
	fightIndex				= 1,
	text							= {
		{ tag = "[Rank]. [Key]" },
		{ tag= "[BarValue:Decimal1]" },
		{ tag = "[(BarValue/TotalBarValue):Percent1]", visible = false },
		count = 3,
	},
	description				= L["Average Instant HPS"],
	onClickType				= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= LH["Healing Done Details"],
	onClickData				= "[Key]",
	favorite					= false,
	defaultColor			= { 0.3, 0.8, 0.5 },
	colorTypeList			= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.activity =
{
	type = "group",
	name = L["Activity"],
	desc = L["Activity Options."],
	args =
	{
		gatherActivity =
		{
			type = "text",
			name = L["Activity Data"],
			desc = L["Gathers information on player activity."],
			get = function() return profile.gatherActivityDataFilter end,
			set = function(v) profile.gatherActivityDataFilter = v AssessmentActivity:OnInitialize() end,
			validate = Assessment.GatherFilterValidate,
		},
		mergePetWithOwner =
		{
			type = "toggle",
			name = L["Merge Pet Activity"],
			desc = L["Merges pets activity with their owner's data."],
			get = function() return profile.mergePetWithOwner end,
			set = function(v) profile.mergePetWithOwner = v end,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentActivity:OnInitialize()
	profile										= self.db.profile
	profileColors							= profile.colors
	gatherActivityDataMethod	= Assessment.GatherFilterList[profile.gatherActivityDataFilter].method
end

-----------------------------------------------------------------------------

function AssessmentActivity:OnEnable()
	self:RegisterParserEvent({ eventType = "Damage" }, 	"OnParserEvent")
	self:RegisterParserEvent({ eventType = "Heal" }, 		"OnParserEvent")
end

function AssessmentActivity:OnDisable()
end

-----------------------------------------------------------------------------

local petToOwnerTable = { pet = "player" }
for i=1,4 do petToOwnerTable["partypet"..i] = "party"..i end
for i=1,40 do petToOwnerTable["raidpet"..i] = "raid"..i end

-----------------------------------------------------------------------------

function AssessmentActivity:OnParserEvent(eventData)
	local sourceName = eventData.sourceName
	if not sourceName or eventData.isFailed then return end
	
	if profile.mergePetWithOwner then
		local sourceOwnerID = petToOwnerTable[eventData.sourceID]
		if sourceOwnerID then
			sourceName = UnitName(sourceOwnerID)
		end
	end
	
	if not gatherActivityDataMethod(sourceName) then
		return
	end

	local time = GetTime()
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.activity = time
		
		local activity = data.activity
		if not activity then
			activity = new()
			data.activity = activity
		end

		local actor = activity[sourceName]
		if actor then
			local dt = time - actor.last
			dt = max(0, min(dt, 3.5))
			
			actor.total = actor.total + dt
			actor.last = time
			
			local eventType = eventData.eventType
			if eventType == "Damage" then
				local damage_dt = time - actor.last_damage
				damage_dt = max(0, min(damage_dt, 3.5))
				
				actor.damage = actor.damage + damage_dt
				actor.last_damage = time
			elseif eventType == "Heal" then
				local healing_dt = time - actor.last_healing
				healing_dt = max(0, min(healing_dt, 3.5))
				
				actor.healing = actor.healing + healing_dt
				actor.last_healing = time
			end
		else
			actor = new()
			actor.total = 0
			actor.damage = 0
			actor.healing = 0
			actor.last = time
			actor.last_healing = time
			actor.last_damage = time
			activity[sourceName] = actor
		end
	end
end

-----------------------------------------------------------------------------
